@@ -1,34 +1,46 @@
+/****************************************************************************
+ * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ *                                                                          *
+ * Permission is hereby granted, free of charge, to any person obtaining a  *
+ * copy of this software and associated documentation files (the            *
+ * "Software"), to deal in the Software without restriction, including      *
+ * without limitation the rights to use, copy, modify, merge, publish,      *
+ * distribute, distribute with modifications, sublicense, and/or sell       *
+ * copies of the Software, and to permit persons to whom the Software is    *
+ * furnished to do so, subject to the following conditions:                 *
+ *                                                                          *
+ * The above copyright notice and this permission notice shall be included  *
+ * in all copies or substantial portions of the Software.                   *
+ *                                                                          *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
+ * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
+ * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
+ * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
+ * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
+ *                                                                          *
+ * Except as contained in this notice, the name(s) of the above copyright   *
+ * holders shall not be used in advertising or otherwise to promote the     *
+ * sale, use or other dealings in this Software without prior written       *
+ * authorization.                                                           *
+ ****************************************************************************/
+
+/****************************************************************************
+ *   Author: Juergen Pfeifer <Juergen.Pfeifer@T-Online.de> 1995,1997        *
+ ****************************************************************************/
 
 /***************************************************************************
-*                            COPYRIGHT NOTICE                              *
-****************************************************************************
-*                ncurses is copyright (C) 1992-1995                        *
-*                          Zeyd M. Ben-Halim                               *
-*                          zmbenhal@netcom.com                             *
-*                          Eric S. Raymond                                 *
-*                          esr@snark.thyrsus.com                           *
-*                                                                          *
-*        Permission is hereby granted to reproduce and distribute ncurses  *
-*        by any means and for any fee, whether alone or as part of a       *
-*        larger distribution, in source or in binary form, PROVIDED        *
-*        this notice is included with any such distribution, and is not    *
-*        removed from any of its header files. Mention of ncurses in any   *
-*        applications linked with it is highly appreciated.                *
-*                                                                          *
-*        ncurses comes AS IS with no warranty, implied or expressed.       *
-*                                                                          *
-***************************************************************************/
-
-/***************************************************************************
-* Module menu_attribs                                                      *
+* Module m_attribs                                                         *
 * Control menus display attributes                                         *
 ***************************************************************************/
 
 #include "menu.priv.h"
-#include <ctype.h>
+
+MODULE_ID("$Id: m_attribs.c,v 1.6 1998/02/11 12:13:49 tom Exp $")
 
 /* Macro to redraw menu if it is posted and changed */
-#define REFRESH_MENU(menu) \
+#define Refresh_Menu(menu) \
    if ( (menu) && ((menu)->status & _POSTED) )\
    {\
       _nc_Draw_Menu( menu );\
@@ -37,30 +49,24 @@
 
 /* "Template" macro to generate a function to set a menus attribute */
 #define GEN_MENU_ATTR_SET_FCT( name ) \
-int set_menu_ ## name (\
-   MENU * menu,\
-   chtype attr\
-)\
+int set_menu_ ## name (MENU * menu, chtype attr)\
 {\
-   if (!(attr==A_NORMAL || (attr & A_ATTRIBUTES)!=0))\
+   if (!(attr==A_NORMAL || (attr & A_ATTRIBUTES)==attr))\
       RETURN(E_BAD_ARGUMENT);\
-   CDEFMENU( menu );\
-   (menu -> name) = attr & A_ATTRIBUTES;\
-   if (menu != &_nc_Default_Menu)\
+   if (menu && ( menu -> name != attr))\
      {\
-	REFRESH_MENU( menu );\
+       (menu -> name) = attr;\
+       Refresh_Menu(menu);\
      }\
+   Normalize_Menu( menu ) -> name = attr;\
    RETURN(E_OK);\
 }
 
 /* "Template" macro to generate a function to get a menus attribute */
 #define GEN_MENU_ATTR_GET_FCT( name ) \
-chtype menu_ ## name (\
-   const MENU * menu\
-)\
+chtype menu_ ## name (const MENU * menu)\
 {\
-   CDEFMENU( menu );\
-   return ( (menu -> name) );\
+   return (Normalize_Menu( menu ) -> name);\
 }
 
 /*---------------------------------------------------------------------------
@@ -72,9 +78,8 @@ chtype menu_ ## name (\
 |                    item ((i.e. where the cursor is), in multi-valued
 |                    menus this is used to highlight the selected items.
 |
-|   Return Values :  E_OK              - on success
+|   Return Values :  E_OK              - success
 |                    E_BAD_ARGUMENT    - an invalid value has been passed   
-|
 +--------------------------------------------------------------------------*/
 GEN_MENU_ATTR_SET_FCT( fore )
 
@@ -87,7 +92,6 @@ GEN_MENU_ATTR_SET_FCT( fore )
 |                    valued menu).   
 |
 |   Return Values :  Attribute value
-|
 +--------------------------------------------------------------------------*/
 GEN_MENU_ATTR_GET_FCT( fore )
 
@@ -98,13 +102,11 @@ GEN_MENU_ATTR_GET_FCT( fore )
 |   Description   :  Set the attribute for selectable but not yet selected
 |                    items.
 |
-|   Return Values :  E_OK             - on success  
+|   Return Values :  E_OK             - success  
 |                    E_BAD_ARGUMENT   - an invalid value has been passed
-|
 +--------------------------------------------------------------------------*/
 GEN_MENU_ATTR_SET_FCT( back )
 
-
 /*---------------------------------------------------------------------------
 |   Facility      :  libnmenu  
 |   Function      :  chtype menu_back(const MENU *menu)
@@ -113,7 +115,6 @@ GEN_MENU_ATTR_SET_FCT( back )
 |                    selected items. 
 |
 |   Return Values :  Attribute value
-|
 +--------------------------------------------------------------------------*/
 GEN_MENU_ATTR_GET_FCT( back )
 
@@ -123,9 +124,8 @@ GEN_MENU_ATTR_GET_FCT( back )
 |   
 |   Description   :  Set the attribute for unselectable items.
 |
-|   Return Values :  E_OK             - on success
+|   Return Values :  E_OK             - success
 |                    E_BAD_ARGUMENT   - an invalid value has been passed    
-|
 +--------------------------------------------------------------------------*/
 GEN_MENU_ATTR_SET_FCT( grey )
 
@@ -136,53 +136,6 @@ GEN_MENU_ATTR_SET_FCT( grey )
 |   Description   :  Return the attribute used for non-selectable items
 |
 |   Return Values :  Attribute value
-|
 +--------------------------------------------------------------------------*/
 GEN_MENU_ATTR_GET_FCT( grey )
-
-/*---------------------------------------------------------------------------
-|   Facility      :  libnmenu  
-|   Function      :  int set_menu_pad(MENU *menu, int pad)
-|   
-|   Description   :  Set the character to be used to separate the item name
-|                    from its description. This must be a printable 
-|                    character.
-|
-|   Return Values :  E_OK              - on success
-|                    E_BAD_ARGUMENT    - an invalid value has been passed
-|
-+--------------------------------------------------------------------------*/
-int set_menu_pad(MENU *menu, int pad)
-{
-    unsigned char padch = (unsigned char)pad;
-
-    if (!isprint(padch))
-	RETURN(E_BAD_ARGUMENT);
-  
-    CDEFMENU( menu );
-    menu->pad = pad;
-  
-    if (menu != &_nc_Default_Menu)
-    {
-	REFRESH_MENU( menu );
-    }
-    RETURN(E_OK);
-}
-
-
-/*---------------------------------------------------------------------------
-|   Facility      :  libnmenu  
-|   Function      :  int menu_pad(const MENU *menu)
-|   
-|   Description   :  Return the value of the padding character
-|
-|   Return Values :  The pad character
-|
-+--------------------------------------------------------------------------*/
-int menu_pad(const MENU * menu)
-{
-    CDEFMENU( menu );
-    return (menu->pad);
-}
-
-/* menu_attribs.c ends here */
+/* m_attribs.c ends here */
