@@ -1,23 +1,35 @@
+/****************************************************************************
+ * Copyright (c) 1998 Free Software Foundation, Inc.                        *
+ *                                                                          *
+ * Permission is hereby granted, free of charge, to any person obtaining a  *
+ * copy of this software and associated documentation files (the            *
+ * "Software"), to deal in the Software without restriction, including      *
+ * without limitation the rights to use, copy, modify, merge, publish,      *
+ * distribute, distribute with modifications, sublicense, and/or sell       *
+ * copies of the Software, and to permit persons to whom the Software is    *
+ * furnished to do so, subject to the following conditions:                 *
+ *                                                                          *
+ * The above copyright notice and this permission notice shall be included  *
+ * in all copies or substantial portions of the Software.                   *
+ *                                                                          *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS  *
+ * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF               *
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.   *
+ * IN NO EVENT SHALL THE ABOVE COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,   *
+ * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR    *
+ * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR    *
+ * THE USE OR OTHER DEALINGS IN THE SOFTWARE.                               *
+ *                                                                          *
+ * Except as contained in this notice, the name(s) of the above copyright   *
+ * holders shall not be used in advertising or otherwise to promote the     *
+ * sale, use or other dealings in this Software without prior written       *
+ * authorization.                                                           *
+ ****************************************************************************/
 
-/***************************************************************************
-*                            COPYRIGHT NOTICE                              *
-****************************************************************************
-*                ncurses is copyright (C) 1992-1995                        *
-*                          Zeyd M. Ben-Halim                               *
-*                          zmbenhal@netcom.com                             *
-*                          Eric S. Raymond                                 *
-*                          esr@snark.thyrsus.com                           *
-*                                                                          *
-*        Permission is hereby granted to reproduce and distribute ncurses  *
-*        by any means and for any fee, whether alone or as part of a       *
-*        larger distribution, in source or in binary form, PROVIDED        *
-*        this notice is included with any such distribution, and is not    *
-*        removed from any of its header files. Mention of ncurses in any   *
-*        applications linked with it is highly appreciated.                *
-*                                                                          *
-*        ncurses comes AS IS with no warranty, implied or expressed.       *
-*                                                                          *
-***************************************************************************/
+/****************************************************************************
+ *  Author: Zeyd M. Ben-Halim <zmbenhal@netcom.com> 1992,1995               *
+ *     and: Eric S. Raymond <esr@snark.thyrsus.com>                         *
+ ****************************************************************************/
 
 
 
@@ -31,17 +43,34 @@
  *
  *	char *captoinfo(n, s, parametrized)
  *
- *	Convert value s for termcap string capabilitynamed n into terminfo
- *	format.  
+ *	Convert value s for termcap string capability named n into terminfo
+ *	format.
+ *
+ *	This code recognizes all the standard 4.4BSD %-escapes:
+ *
+ *	%%       output `%'
+ *	%d       output value as in printf %d
+ *	%2       output value as in printf %2d
+ *	%3       output value as in printf %3d
+ *	%.       output value as in printf %c
+ *	%+x      add x to value, then do %.
+ *	%>xy     if value > x then add y, no output
+ *	%r       reverse order of two parameters, no output
+ *	%i       increment by one, no output
+ *	%n       exclusive-or all parameters with 0140 (Datamedia 2500)
+ *	%B       BCD (16*(value/10)) + (value%10), no output
+ *	%D       Reverse coding (value - 2*(value%16)), no output (Delta Data).
+ *
+ *	Also, %02 and %03 are accepted as synonyms for %2 and %3.
  *
  *	Besides all the standard termcap escapes, this translator understands
  *	the following extended escapes:
  *
  *	used by GNU Emacs termcap libraries
- *		%a[+*-/=][cp]x	GNU arithmetic. 
- *		%m	  	xor the first two parameters by 0177
- *		%b	  	backup to previous parameter
- *		%f	  	skip this parameter
+ *		%a[+*-/=][cp]x	GNU arithmetic.
+ *		%m		xor the first two parameters by 0177
+ *		%b		backup to previous parameter
+ *		%f		skip this parameter
  *
  *	used by the University of Waterloo (MFCF) termcap libraries
  *		%-x	 subtract parameter FROM char x and output it as a char
@@ -55,28 +84,20 @@
  *	the Waterloo %s conflicts with the way terminfo uses %s in strings for
  *	function programming.
  *
- * 	Note the two definitions of %a: the GNU definition is translated if the
+ *	Note the two definitions of %a: the GNU definition is translated if the
  *	characters after the 'a' are valid for it, otherwise the UW definition
  *	is translated.
  */
 
-#include "curses.priv.h"
+#include <curses.priv.h>
 
-#include <stdlib.h>
-#include <string.h>
 #include <ctype.h>
-#include <unctrl.h>
-#include "tic.h"
+#include <tic.h>
 
-#ifndef MAX_PUSHED
-/* maximum # of parameters that can be pushed onto the stack */
-#define MAX_PUSHED 16
-#endif
+MODULE_ID("$Id: captoinfo.c,v 1.17 1998/02/11 12:13:54 tom Exp $")
 
-#ifndef MAX_ENTRY
-/* maximum characters in a translated capability */
-#define MAX_ENTRY	80
-#endif
+#define MAX_PUSHED	16	/* max # args we can push onto the stack */
+#define MAX_ENTRY	2048	/* maximum chars in a translated capability */
 
 static int stack[MAX_PUSHED];	/* the stack */
 static int stackptr;		/* the next empty place on the stack */
@@ -87,6 +108,40 @@ static int seenr;		/* seen a %r */
 static int param;		/* current parameter */
 static char *dp;		/* pointer to end of the converted string */
 
+static char  *my_string;
+static size_t my_length;
+
+static char *init_string(void)
+/* initialize 'my_string', 'my_length' */
+{
+	if (my_string == 0)
+		my_string = malloc(my_length = 256);
+	if (my_string == 0)
+	    _nc_err_abort("Out of memory");
+
+	*my_string = '\0';
+	return my_string;
+}
+
+static char *save_string(char *d, const char *const s)
+{
+	size_t have = (d - my_string);
+	size_t need = have + strlen(s) + 2;
+	if (need > my_length) {
+		my_string = realloc(my_string, my_length = (need + need));
+		d = my_string + have;
+	}
+	(void) strcpy(d, s);
+	return d + strlen(d);
+}
+
+static inline char *save_char(char *s, char c)
+{
+	static char temp[2];
+	temp[0] = c;
+	return save_string(s, temp);
+}
+
 static void push(void)
 /* push onstack on to the stack */
 {
@@ -99,7 +154,7 @@ static void push(void)
 static void pop(void)
 /* pop the top of the stack into onstack */
 {
-    if (stackptr == 0) 
+    if (stackptr == 0)
 	if (onstack == 0)
 	    _nc_warning("I'm confused");
 	else
@@ -109,7 +164,7 @@ static void pop(void)
     param++;
 }
 
-static int cvtchar(register char *sp)
+static int cvtchar(register const char *sp)
 /* convert a character to a terminfo push */
 {
     unsigned char c = 0;
@@ -161,7 +216,7 @@ static int cvtchar(register char *sp)
 	if (c > 99)
 	    *dp++ = c / 100 + '0';
 	if (c > 9)
-	    *dp++ = (c / 10) % 10 + '0';
+	    *dp++ = ((int)(c / 10)) % 10 + '0';
 	*dp++ = c % 10 + '0';
 	*dp++ = '}';
     }
@@ -191,7 +246,7 @@ static void getparm(int parm, int n)
 		push();
 
 	onstack = parm;
-	
+
 	while(n--) {		/* %p0 */
 		*dp++ = '%'; *dp++ = 'p'; *dp++ = '0' + parm;
 	}
@@ -200,7 +255,7 @@ static void getparm(int parm, int n)
 		*dp++ = '%'; *dp++ = '{'; *dp++ = '9'; *dp++ = '6'; *dp++ = '}';
 		*dp++ = '%'; *dp++ = '^';
 	}
-	
+
 	if (seenm && parm < 3) {	/* %{127}%^ */
 		*dp++ = '%'; *dp++ = '{'; *dp++ = '1'; *dp++ = '2'; *dp++ = '7';
 		*dp++ = '}'; *dp++ = '%'; *dp++ = '^';
@@ -209,12 +264,12 @@ static void getparm(int parm, int n)
 
 char *_nc_captoinfo(
 /* convert a termcap string to terminfo format */
-register char *cap,	/* relevant terminfo capability index */
-register char *s,	/* string value of the capability */
-bool parametrized)	/* do % translations? */	
+register const char *cap,	/* relevant terminfo capability index */
+register const char *s,		/* string value of the capability */
+int const parametrized)		/* do % translations if 1, pad translations if >=0 */
 {
     static char line[MAX_ENTRY];
-    char *capstart;
+    const char *capstart;
 
     stackptr = 0;
     onstack = 0;
@@ -225,9 +280,11 @@ bool parametrized)	/* do % translations? */
 
     dp = line;
 
-    /* skip the initial padding */
-    capstart = (char *)NULL;
-    if (isdigit(*s))
+    /* skip the initial padding (if we haven't been told not to) */
+    capstart = 0;
+    if (s == 0)
+	s = "";
+    if (parametrized >= 0 && isdigit(*s))
 	for (capstart = s; ; s++)
 	    if (!(isdigit(*s) || *s == '*' || *s == '.'))
 		break;
@@ -236,7 +293,7 @@ bool parametrized)	/* do % translations? */
 	switch(*s) {
 	case '%':
 	    s++;
-	    if (!parametrized) {
+	    if (parametrized < 1) {
 		*dp++ = '%';
 		break;
 	    }
@@ -244,21 +301,21 @@ bool parametrized)	/* do % translations? */
 	    case '%': *dp++ = '%'; break;
 	    case 'r':
 		if (seenr++ == 1) {
-		    _nc_warning("saw %%r twice");
+		    _nc_warning("saw %%r twice in %s", cap);
 		}
 		break;
 	    case 'm':
 		if (seenm++ == 1) {
-		    _nc_warning("saw %%m twice");
+		    _nc_warning("saw %%m twice in %s", cap);
 		}
 		break;
 	    case 'n':
 		if (seenn++ == 1) {
-		    _nc_warning("saw %%n twice");
+		    _nc_warning("saw %%n twice in %s", cap);
 		}
 		break;
 	    case 'i': *dp++ = '%'; *dp++ = 'i'; break;
-	    case '6': 
+	    case '6':
 	    case 'B':
 		getparm(param, 2);
 		/* %{6}%*%+ */
@@ -277,7 +334,7 @@ bool parametrized)	/* do % translations? */
 	    case '>':
 		getparm(param, 2);
 		/* %?%{x}%>%t%{y}%+%; */
-		*dp++ = '%'; *dp++ = '?'; 
+		*dp++ = '%'; *dp++ = '?';
 		s += cvtchar(s);
 		*dp++ = '%'; *dp++ = '>';
 		*dp++ = '%'; *dp++ = 't';
@@ -365,13 +422,19 @@ bool parametrized)	/* do % translations? */
 		*dp++ = '%'; *dp++ = 'c';
 		pop();
 		break;
+	    case '0':	/* not clear any of the historical termcaps did this */
+	        if (*s == '3')
+		    goto see03;
+		else if (*s != '2')
+		    goto invalid;
+		/* FALLTHRU */
 	    case '2':
 		getparm(param, 1);
 		*dp++ = '%'; /* *dp++ = '0'; */
 		*dp++ = '2'; *dp++ = 'd';
 		pop();
 		break;
-	    case '3':
+	    case '3': see03:
 		getparm(param, 1);
 		*dp++ = '%'; /* *dp++ = '0'; */
 		*dp++ = '3'; *dp++ = 'd';
@@ -392,11 +455,11 @@ bool parametrized)	/* do % translations? */
 		*dp++ = '%';
 		*dp++ = '\\';
 		break;
-	    default:
+	    default: invalid:
 		*dp++ = '%';
 		s--;
-		_nc_warning("'%s' unknown %% code %s",
-			cap, _tracechar(*s));
+		_nc_warning("unknown %% code %s in %s",
+			_tracechar(*s), cap);
 		break;
 	    }
 	    break;
@@ -488,25 +551,18 @@ bool parametrized)	/* do % translations? */
  *     %m       exclusive-or all parameters with 0177 (not in 4.4BSD)
  */
 
-static	char	buffer[256];
-static	char	*bufptr;
-
 char *_nc_infotocap(
 /* convert a terminfo string to termcap format */
-register char *cap,	/* relevant termcap capability index */
-register char *str,	/* string value of the capability */
-bool parametrized)	/* do % translations? */
+register const char *cap GCC_UNUSED, /* relevant termcap capability index */
+register const char *str,	/* string value of the capability */
+int const parametrized)		/* do % translations if 1, pad translations if >=0 */
 {
     int	seenone = 0, seentwo = 0, saw_m = 0, saw_n = 0;
-    char *padding, ch1 = 0, ch2 = 0;
-
-    /*
-     * Yes, this code is rather ad-hoc.  It passes the only important
-     * test, which is that it correctly back-translates a terminfo version
-     * of the historical termcap database -- esr.
-     */
-    memset(buffer, '\0', sizeof(buffer));
-    bufptr = buffer;
+    const char *padding;
+    const char *trimmed = 0;
+    char ch1 = 0, ch2 = 0;
+    char *bufptr = init_string();
+    char temp[256];
 
     /* we may have to move some trailing mandatory padding up front */
     padding = str + strlen(str) - 1;
@@ -516,21 +572,21 @@ bool parametrized)	/* do % translations? */
 	while (isdigit(*padding) || *padding == '.' || *padding == '*')
 	    padding--;
 	if (*padding == '<' && *--padding == '$')
-	    *padding = '\0';
+	    trimmed = padding;
 	padding += 2;
 
 	while (isdigit(*padding) || *padding == '.' || *padding == '*')
-	    *bufptr++ = *padding++;
+	    bufptr = save_char(bufptr, *padding++);
     }
 
-    for (; *str; str++)
+    for (; *str && str != trimmed; str++)
     {
 	int	c1, c2;
 	char	*cp;
 
 	if (str[0] == '\\' && (str[1] == '^' || str[1] == ','))
 	{
-	    *(bufptr++) = *++str;
+	    bufptr = save_char(bufptr, *++str);
 	}
 	else if (str[0] == '$' && str[1] == '<')	/* discard padding */
 	{
@@ -538,75 +594,75 @@ bool parametrized)	/* do % translations? */
 	    while (isdigit(*str) || *str == '.' || *str == '*' || *str == '/' || *str == '>')
 		str++;
 	    --str;
-	}	
-	else if (*str != '%' || !parametrized)
-	    *(bufptr++) = *str;
+	}
+	else if (*str != '%' || (parametrized < 1))
+	    bufptr = save_char(bufptr, *str);
 	else if (sscanf(str, "%%?%%{%d}%%>%%t%%{%d}%%+%%;", &c1,&c2) == 2)
 	{
 	    str = strchr(str, ';');
-	    (void) sprintf(bufptr, "%%>");
-	    (void) strcat(bufptr, unctrl(c1));
-	    (void) strcat(bufptr, unctrl(c2));
-	    bufptr += strlen(bufptr);
+	    (void) sprintf(temp, "%%>");
+	    (void) strcat(temp, unctrl(c1));
+	    (void) strcat(temp, unctrl(c2));
+	    bufptr = save_string(bufptr, temp);
 	}
 	else if (sscanf(str, "%%?%%{%d}%%>%%t%%'%c'%%+%%;", &c1,&ch2) == 2)
 	{
 	    str = strchr(str, ';');
-	    (void) sprintf(bufptr, "%%>%s%c", unctrl(c1), ch2);
-	    bufptr += strlen(bufptr);
+	    (void) sprintf(temp, "%%>%s%c", unctrl(c1), ch2);
+	    bufptr = save_string(bufptr, temp);
 	}
 	else if (sscanf(str, "%%?%%'%c'%%>%%t%%{%d}%%+%%;", &ch1,&c2) == 2)
 	{
 	    str = strchr(str, ';');
-	    (void) sprintf(bufptr, "%%>%c%c", ch1, c2);
-	    bufptr += strlen(bufptr);
+	    (void) sprintf(temp, "%%>%c%c", ch1, c2);
+	    bufptr = save_string(bufptr, temp);
 	}
 	else if (sscanf(str, "%%?%%'%c'%%>%%t%%'%c'%%+%%;", &ch1, &ch2) == 2)
 	{
 	    str = strchr(str, ';');
-	    (void) sprintf(bufptr, "%%>%c%c", ch1, ch2);
-	    bufptr += strlen(bufptr);
+	    (void) sprintf(temp, "%%>%c%c", ch1, ch2);
+	    bufptr = save_string(bufptr, temp);
 	}
 	else if (strncmp(str, "%{6}%*%+", 8) == 0)
 	{
 	    str += 7;
-	    (void) sprintf(bufptr, "%%B");
-	    bufptr += strlen(bufptr);
+	    (void) sprintf(temp, "%%B");
+	    bufptr = save_string(bufptr, temp);
 	}
 	else if ((sscanf(str, "%%{%d}%%+%%c", &c1) == 1
 		  || sscanf(str, "%%'%c'%%+%%c", &ch1) == 1)
 		 && (cp = strchr(str, '+')))
 	{
 	    str = cp + 2;
-	    *bufptr++ = '%';
-	    *bufptr++ = '+';
+	    bufptr = save_char(bufptr, '%');
+	    bufptr = save_char(bufptr, '+');
 
 	    if (ch1)
 		c1 = ch1;
 	    if (is7bits(c1) && isprint(c1))
-		*bufptr++ = c1;
+		bufptr = save_char(bufptr, (char)c1);
 	    else
 	    {
 		if (c1 == (c1 & 0x1f)) /* iscntrl() returns T on 255 */
-		    (void) strcat(bufptr, unctrl(c1));
+		    (void) strcpy(temp, unctrl(c1));
 		else
-		    (void) sprintf(bufptr, "\\%03o", c1);
-		bufptr += strlen(bufptr);
+		    (void) sprintf(temp, "\\%03o", c1);
+		bufptr = save_string(bufptr, temp);
 	    }
 	}
 	else if (strncmp(str, "%{2}%*%-", 8) == 0)
 	{
 	    str += 7;
-	    (void) sprintf(bufptr, "%%D");
-	    bufptr += strlen(bufptr);
+	    (void) sprintf(temp, "%%D");
+	    bufptr = save_string(bufptr, temp);
 	}
 	else if (strncmp(str, "%{96}%^", 7) == 0)
 	{
 	    str += 6;
 	    if (saw_m++ == 0)
 	    {
-		(void) sprintf(bufptr, "%%n");
-		bufptr += strlen(bufptr);
+		(void) sprintf(temp, "%%n");
+		bufptr = save_string(bufptr, temp);
 	    }
 	}
 	else if (strncmp(str, "%{127}%^", 8) == 0)
@@ -614,8 +670,8 @@ bool parametrized)	/* do % translations? */
 	    str += 7;
 	    if (saw_n++ == 0)
 	    {
-		(void) sprintf(bufptr, "%%m");
-		bufptr += strlen(bufptr);
+		(void) sprintf(temp, "%%m");
+		bufptr = save_string(bufptr, temp);
 	    }
 	}
 	else
@@ -623,7 +679,7 @@ bool parametrized)	/* do % translations? */
 	    str++;
 	    switch (*str) {
 	    case '%':
-		*(bufptr++) = '%';
+		bufptr = save_char(bufptr, '%');
 		break;
 
 	    case '0':
@@ -636,24 +692,25 @@ bool parametrized)	/* do % translations? */
 	    case '7':
 	    case '8':
 	    case '9':
-		*(bufptr++) = '%';
+		bufptr = save_char(bufptr, '%');
 		while (isdigit(*str))
-		    *bufptr++ = *str++;
+		    bufptr = save_char(bufptr, *str++);
 		if (*str == 'd')
 		    str++;
 		else
-		    _nc_warning("numeric prefix is missing trailing d");
+		    _nc_warning("numeric prefix is missing trailing d in %s",
+				cap);
 		--str;
 		break;
 
 	    case 'd':
-		*(bufptr++) = '%';
-		*(bufptr++) = 'd';
+		bufptr = save_char(bufptr, '%');
+		bufptr = save_char(bufptr, 'd');
 		break;
 
 	    case 'c':
-		*(bufptr++) = '%';
-		*(bufptr++) = '.';
+		bufptr = save_char(bufptr, '%');
+		bufptr = save_char(bufptr, '.');
 		break;
 
 	    /*
@@ -662,8 +719,8 @@ bool parametrized)	/* do % translations? */
 	     * termcap notation.
 	     */
 	    case 's':
-		*(bufptr++) = '%';
-		*(bufptr++) = 's';
+		bufptr = save_char(bufptr, '%');
+		bufptr = save_char(bufptr, 's');
 		break;
 
 	    case 'p':
@@ -674,42 +731,35 @@ bool parametrized)	/* do % translations? */
 		{
 		    if (!seenone && !seentwo)
 		    {
-			*(bufptr++) = '%';
-			*(bufptr++) = 'r';
+			bufptr = save_char(bufptr, '%');
+			bufptr = save_char(bufptr, 'r');
 			seentwo++;
 		    }
 		}
 		else if (*str >= '3')
-		    return((char *)NULL);
+		    return(0);
 		break;
 
 	    case 'i':
-		*(bufptr++) = '%';
-		*(bufptr++) = 'i';
+		bufptr = save_char(bufptr, '%');
+		bufptr = save_char(bufptr, 'i');
 		break;
 
 	    default:
-		return((char *)NULL);
+		return(0);
 
 	    } /* endswitch (*str) */
 	} /* endelse (*str == '%') */
-	
+
 	if (*str == '\0')
 	    break;
 
     } /* endwhile (*str) */
 
-    *bufptr = '\0';
-    return(buffer);
-
+    return(my_string);
 }
 
 #ifdef MAIN
-#if HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#include <stdio.h>
-#include <stdlib.h>
 
 int curr_line;
 
@@ -731,20 +781,20 @@ int main(int argc, char *argv[])
 	char	buf[BUFSIZ];
 
 	++curr_line;
-	if (fgets(buf, sizeof(buf), stdin) == (char *)NULL)
+	if (fgets(buf, sizeof(buf), stdin) == 0)
 	    break;
 	buf[strlen(buf) - 1] = '\0';
 	_nc_set_source(buf);
 
 	if (tc)
 	{
-	    char	*cp = infotocap("to termcap", buf, 1);
+	    char	*cp = _nc_infotocap("to termcap", buf, 1);
 
 	    if (cp)
 		(void) fputs(cp, stdout);
 	}
 	else
-	    (void) fputs(captoinfo("to terminfo", buf, 1), stdout);
+	    (void) fputs(_nc_captoinfo("to terminfo", buf, 1), stdout);
 	(void) putchar('\n');
     }
     return(0);
