@@ -1,23 +1,43 @@
-#include <stdio.h>
-#include <stdlib.h>
-#include <sys/types.h>
+/*
+ * $Id: firework.c,v 1.11 1997/11/01 19:36:25 tom Exp $
+ */
+#include <test.priv.h>
+
+#include <term.h>	/* for tparm() */
+
 #include <signal.h>
-#include <curses.h>
 #include <ctype.h>
 #include <time.h>
 
-static int get_colour(void);
+static int get_colour(chtype *);
 static void explode(int row, int col);
 static void showit(void);
+static RETSIGTYPE onsig(int sig);
 
-int main(int argc, char *argv[])
+static int my_bg = COLOR_BLACK;
+
+int main(
+	int argc GCC_UNUSED,
+	char *argv[] GCC_UNUSED)
 {
+int j;
 int start,end,row,diff,flag = 0,direction;
 unsigned seed;
 
-       initscr();
-       if (has_colors())
-          start_color();
+	for (j=SIGHUP;j<=SIGTERM;j++)
+		if (signal(j,SIG_IGN)!=SIG_IGN)
+			signal(j,onsig);
+
+	initscr();
+	if (has_colors()) {
+		start_color();
+#ifdef NCURSES_VERSION
+		if (use_default_colors() == OK)
+			my_bg = -1;
+#endif
+	}
+	curs_set(0);
+
        seed = time((time_t *)0);
        srand(seed);
        cbreak();
@@ -52,22 +72,31 @@ unsigned seed;
        }
 }
 
+static RETSIGTYPE
+onsig(int n GCC_UNUSED)
+{
+    curs_set(1);
+    endwin();
+    exit(EXIT_FAILURE);
+}
+
 static
 void explode(int row, int col)
 {
+       chtype bold;
        erase();
        mvprintw(row,col,"-");
        showit();
 
-       init_pair(1,get_colour(),COLOR_BLACK);
-       attrset(COLOR_PAIR(1));
+       init_pair(1,get_colour(&bold),my_bg);
+       attrset(COLOR_PAIR(1) | bold);
        mvprintw(row-1,col-1," - ");
        mvprintw(row,col-1,"-+-");
        mvprintw(row+1,col-1," - ");
        showit();
 
-       init_pair(1,get_colour(),COLOR_BLACK);
-       attrset(COLOR_PAIR(1));
+       init_pair(1,get_colour(&bold),my_bg);
+       attrset(COLOR_PAIR(1) | bold);
        mvprintw(row-2,col-2," --- ");
        mvprintw(row-1,col-2,"-+++-");
        mvprintw(row,  col-2,"-+#+-");
@@ -75,8 +104,8 @@ void explode(int row, int col)
        mvprintw(row+2,col-2," --- ");
        showit();
 
-       init_pair(1,get_colour(),COLOR_BLACK);
-       attrset(COLOR_PAIR(1));
+       init_pair(1,get_colour(&bold),my_bg);
+       attrset(COLOR_PAIR(1) | bold);
        mvprintw(row-2,col-2," +++ ");
        mvprintw(row-1,col-2,"++#++");
        mvprintw(row,  col-2,"+# #+");
@@ -84,8 +113,8 @@ void explode(int row, int col)
        mvprintw(row+2,col-2," +++ ");
        showit();
 
-       init_pair(1,get_colour(),COLOR_BLACK);
-       attrset(COLOR_PAIR(1));
+       init_pair(1,get_colour(&bold),my_bg);
+       attrset(COLOR_PAIR(1) | bold);
        mvprintw(row-2,col-2,"  #  ");
        mvprintw(row-1,col-2,"## ##");
        mvprintw(row,  col-2,"#   #");
@@ -93,8 +122,8 @@ void explode(int row, int col)
        mvprintw(row+2,col-2,"  #  ");
        showit();
 
-       init_pair(1,get_colour(),COLOR_BLACK);
-       attrset(COLOR_PAIR(1));
+       init_pair(1,get_colour(&bold),my_bg);
+       attrset(COLOR_PAIR(1) | bold);
        mvprintw(row-2,col-2," # # ");
        mvprintw(row-1,col-2,"#   #");
        mvprintw(row,  col-2,"     ");
@@ -104,15 +133,17 @@ void explode(int row, int col)
 }
 
 static
-int get_colour(void)
+int get_colour(chtype *bold)
 {
- int attr;
-       attr = (rand() % 16)+1;
-       if (attr == 1 || attr == 9)
-          attr = COLOR_RED;
-       if (attr > 8)
-          attr |= A_BOLD;
-       return(attr);
+	int attr;
+	attr = (rand() % 16) + 1;
+
+	*bold = A_NORMAL;
+	if (attr > 8) {
+		*bold = A_BOLD;
+		attr &= 7;
+	}
+	return(attr);
 }
 
 static void
