@@ -1,28 +1,27 @@
-/* $Header: /src/usr.local/lib/ncurses-1.9.0/test/RCS/newdemo.c,v 1.1 1995/04/21 16:39:03 djm Exp djm $
- *
+/*
  *  newdemo.c	-	A demo program using PDCurses. The program illustrate
  *  	 		the use of colours for text output.
+ *
+ * $Id: newdemo.c,v 1.17 1997/09/20 15:11:26 tom Exp $
  */
 
-#include <stdio.h>
+#include <test.priv.h>
+
 #include <signal.h>
 #include <time.h>
 #include <string.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <curses.h>
 
-int SubWinTest(WINDOW *win);
-int WaitForUser(void);
-int BouncingBalls(WINDOW *win);
-void trap(int);
+static int SubWinTest(WINDOW *win);
+static int WaitForUser(WINDOW *win);
+static int BouncingBalls(WINDOW *win);
+static RETSIGTYPE trap(int);
 
 #define delay_output(x) napms(x)
 
 /*
  *  The Australian map
  */
-char    *AusMap[16] =
+const char *AusMap[16] =
 {
     "           A           A ",
     "    N.T. AAAAA       AAAA ",
@@ -43,7 +42,7 @@ char    *AusMap[16] =
  */
 #define NMESSAGES   6
 
-char    *messages[] =
+NCURSES_CONST char *messages[] =
 {
     "Hello from the Land Down Under",
     "The Land of crocs. and a big Red Rock",
@@ -58,13 +57,17 @@ char    *messages[] =
  *  Main driver
  */
 int
-main(int argc, char **argv)
+main(
+	int argc GCC_UNUSED,
+	char *argv[] GCC_UNUSED)
 {
 WINDOW  *win;
-int     w, x, y, i, j, c, len;
-char    buffer[80], *message;
+int     w, x, y, i, j, k;
+char    buffer[200];
+const char *message;
 int     width, height;
 chtype  save[80];
+chtype  c;
 
     initscr();
     start_color();
@@ -93,7 +96,7 @@ chtype  save[80];
         for(i=0; i < 5000; ++i)
         {   x = rand() % (width-2)  + 1;
             y = rand() % (height-2) + 1;
-            mvwaddch(win, y, x, (chtype)c);
+            mvwaddch(win, y, x, c);
             wrefresh(win);
             nodelay(win,TRUE);
             if (wgetch(win) != ERR)
@@ -108,6 +111,7 @@ chtype  save[80];
         SubWinTest(win);
                                 /* Erase and draw green window */
         init_pair(4,COLOR_YELLOW,COLOR_GREEN);
+        wbkgd(win, COLOR_PAIR(4) | A_BOLD);
         wattrset(win, COLOR_PAIR(4) | A_BOLD);
         werase(win);
         wrefresh(win);
@@ -133,37 +137,31 @@ chtype  save[80];
 				/* Draw running messages */
 	init_pair(6,COLOR_YELLOW,COLOR_WHITE);
 	wattrset(win, COLOR_PAIR(6));
-	message = messages[0];
-	len = strlen(message);
-	j = 0;
-	i = 2;
+	message = messages[j = 0];
+	i = 1;
 	w = width-2;
-        while(j < NMESSAGES)
-        {   strncpy(buffer, message, (size_t)(w - i));
-            buffer[w-i] = 0;
-	    mvwaddstr(win, height/2, i, buffer);
-            if(w - i < len)
-            {   memset(buffer, ' ', (size_t)i);
-                strcpy(buffer, message + (w - i));
-                buffer[strlen(buffer)]   = ' ';
-                buffer[i-2] = '\0';
-                mvwaddstr(win, height/2, 2, buffer);
+	strcpy(buffer, message);
+        while(j < NMESSAGES) {
+	    while ((int)strlen(buffer) < w) {
+		strcat(buffer, " ... ");
+	        strcat(buffer, messages[++j % NMESSAGES]);
 	    }
+
+	    if (i < w)
+	        mvwaddnstr(win, height/2, w - i, buffer, i);
+            else
+	        mvwaddnstr(win, height/2, 1, buffer, w);
+
             wrefresh(win);
             nodelay(win,TRUE);
             if (wgetch(win) != ERR)
             {   flushinp();
 		break;
             }
-            mvwaddch(win, height/2, i, ' ');
-            i = ++i % w;
-            if(i < 2)
-            {   message = messages[++j%NMESSAGES];
-                memset(buffer, ' ', (size_t)(w-2));
-		buffer[w-2] = 0;
-                mvwaddstr(win, height/2, 2, buffer);
-                i = 2;
-            }
+	    if (i++ >= w) {
+	        for (k = 0; (buffer[k] = buffer[k+1]) != '\0'; k++)
+	    	    ;
+	    }
 	    delay_output(100);
         }
 
@@ -172,10 +170,13 @@ chtype  save[80];
         init_pair(7,COLOR_RED,COLOR_GREEN);
         wattron(win, COLOR_PAIR(7));
 	for(i=2; i < width - 4; ++i)
-        {   c = mvwinch(win, 4, i);
-            save[j++] = c;
-            c = c & 0x7f;
-	    mvwaddch(win, 4, i, (chtype)c);
+        {
+            k = mvwinch(win, 4, i);
+	    if (k == ERR)
+	    	break;
+            save[j++] = c = k;
+            c &= A_CHARTEXT;
+	    mvwaddch(win, 4, i, c);
         }
         wrefresh(win);
 
@@ -185,7 +186,7 @@ chtype  save[80];
 	mvwaddstr(win, i, 5, " Type a key to continue or 'Q' to quit ");
         wrefresh(win);
 
-        if(WaitForUser() == 1)
+        if(WaitForUser(win) == 1)
 	    break;
 
         j = 0;                  /* Restore the old line */
@@ -199,7 +200,7 @@ chtype  save[80];
         wattrset(win, COLOR_PAIR(5));
 	mvwaddstr(win, i, 5, " Type a key to continue or 'Q' to quit ");
         wrefresh(win);
-        if(WaitForUser() == 1)
+        if(WaitForUser(win) == 1)
             break;
     }
     endwin();
@@ -209,16 +210,14 @@ chtype  save[80];
 /*
  * Test sub windows
  */
-int
+static int
 SubWinTest(WINDOW *win)
 {
 int     w, h, sw, sh, bx, by;
 WINDOW  *swin1, *swin2, *swin3;
 
-    w  = win->_maxx;
-    h  = win->_maxy;
-    bx = win->_begx;
-    by = win->_begy;
+    getmaxyx(win, h,  w);
+    getbegyx(win, by, bx);
     sw = w / 3;
     sh = h / 3;
     if((swin1 = subwin(win, sh, sw, by+3, bx+5)) == NULL)
@@ -249,14 +248,14 @@ WINDOW  *swin1, *swin2, *swin3;
     delwin(swin1);
     delwin(swin2);
     delwin(swin3);
-    WaitForUser();
+    WaitForUser(win);
     return  0;
 }
 
 /*
  *  Bouncing balls
  */
-int
+static int
 BouncingBalls(WINDOW *win)
 {
 int	w, h;
@@ -264,8 +263,7 @@ int     x1, y1, xd1, yd1;
 int     x2, y2, xd2, yd2;
 int     x3, y3, xd3, yd3;
 
-    w    = win->_maxx;
-    h    = win->_maxy;
+    getmaxyx(win, h, w);
     x1   = 2 + rand() % (w - 4);
     y1   = 2 + rand() % (h - 4);
     x2   = 2 + rand() % (w - 4);
@@ -317,16 +315,16 @@ int     x3, y3, xd3, yd3;
 /*
  *  Wait for user
  */
-int WaitForUser(void)
+static int WaitForUser(WINDOW *win)
 {
  time_t  t;
  chtype key;
 
- nodelay(stdscr,TRUE);
+ nodelay(win,TRUE);
  t = time((time_t *)0);
  while(1)
    {
-    if ((key = getch()) != ERR)
+    if ((int)(key = wgetch(win)) != ERR)
       {
        if (key  == 'q' || key == 'Q')
           return  1;
@@ -341,10 +339,10 @@ int WaitForUser(void)
 /*
  *  Trap interrupt
  */
-void trap(int sig)
+static RETSIGTYPE trap(int sig GCC_UNUSED)
 {
     endwin();
-    exit(sig);
+    exit(EXIT_FAILURE);
 }
 
 /*  End of DEMO.C */
