@@ -33,10 +33,14 @@ Options:
   code.  You can use -T to set the worm move interval over which movement
   traces will be dumped.  The program stops and waits for one character of
   input at the beginning and end of the interval.
+
+  $Id: worm.c,v 1.21 1998/01/30 10:17:59 tom Exp $
 */
 
-#include <curses.h>
-#include <stdlib.h>
+#include <test.priv.h>
+
+#include <term.h>	/* for tparm() */
+
 #include <signal.h>
 
 #define cursor(col,row) move(row,col)
@@ -45,8 +49,8 @@ short *ref[128];
 static chtype flavor[]={
     'O' , '*', '#', '$', '%', '0', '@',
 };
-#define MAXWORMS	sizeof(flavor)
-static short xinc[]={
+#define MAXWORMS	(sizeof(flavor)/sizeof(chtype))
+static const short xinc[]={
      1,  1,  1,  0, -1, -1, -1,  0
 }, yinc[]={
     -1,  0,  1,  1,  1,  0, -1, -1
@@ -56,14 +60,14 @@ static struct worm {
     short *xpos, *ypos;
 } worm[40];
 
-static char *field;
+static const char *field;
 static int length=16, number=3;
 static chtype trail=' ';
 
 #ifdef TRACE
 int generation, trace_start, trace_end, singlestep;
 #endif /* TRACE */
-static struct options {
+static const struct options {
     int nopts;
     int opts[3];
 } normal[8]={
@@ -149,8 +153,8 @@ static struct options {
     { 0, { 0, 0, 0 } }
 };
 
-void onsig(int sig);
-float ranf(void);
+static RETSIGTYPE onsig(int sig);
+static float ranf(void);
 
 int
 main(int argc, char *argv[])
@@ -158,7 +162,7 @@ main(int argc, char *argv[])
 int x, y;
 int n;
 struct worm *w;
-struct options *op;
+const struct options *op;
 int h;
 short *ip;
 int last, bottom;
@@ -175,14 +179,14 @@ int last, bottom;
 		    if (++x==argc) goto usage;
 		    if ((length=atoi(argv[x]))<2||length>1024) {
 				fprintf(stderr,"%s: Invalid length\n",*argv);
-				exit(1);
+				return EXIT_FAILURE;
 		    }
 		    break;
 		case 'n':
 		    if (++x==argc) goto usage;
 		    if ((number=atoi(argv[x]))<1||number>40) {
 				fprintf(stderr,"%s: Invalid number of worms\n",*argv);
-				exit(1);
+				return EXIT_FAILURE;
 		    }
 		    break;
 		case 't':
@@ -197,47 +201,53 @@ int last, bottom;
 		    trace_end   = atoi(argv[++x]);
 		    break;
 		case 'N':
-		    no_optimize = TRUE;		/* declared by ncurses */
+		    _nc_optimize_enable ^= OPTIMIZE_ALL;		/* declared by ncurses */
 		    break;
 #endif /* TRACE */
 		default:
 		usage:
 		    fprintf(stderr, "usage: %s [-field] [-length #] [-number #] [-trail]\n",*argv);
-		    exit(1);
+		    return EXIT_FAILURE;
 		    break;
 		}
     }
 
     signal(SIGINT, onsig);
     initscr();
-#ifdef TRACE
     noecho();
     cbreak();
-#endif /* TRACE */
     nonl();
+
+    curs_set(0);
+
     bottom = LINES-1;
     last = COLS-1;
 
 #ifdef A_COLOR
     if (has_colors())
     {
+	int bg = COLOR_BLACK;
 	start_color();
+#ifdef NCURSES_VERSION
+	if (use_default_colors() == OK)
+	    bg = -1;
+#endif
  
-	init_pair(COLOR_GREEN, COLOR_GREEN, COLOR_BLACK);
-	init_pair(COLOR_RED, COLOR_RED, COLOR_BLACK);
-	init_pair(COLOR_CYAN, COLOR_CYAN, COLOR_BLACK);
-	init_pair(COLOR_WHITE, COLOR_WHITE, COLOR_BLACK);
-	init_pair(COLOR_MAGENTA, COLOR_MAGENTA, COLOR_BLACK);
-	init_pair(COLOR_BLUE, COLOR_BLUE, COLOR_BLACK);
-	init_pair(COLOR_YELLOW, COLOR_YELLOW, COLOR_BLACK);
+	init_pair(COLOR_GREEN,   COLOR_GREEN,   bg);
+	init_pair(COLOR_RED,     COLOR_RED,     bg);
+	init_pair(COLOR_CYAN,    COLOR_CYAN,    bg);
+	init_pair(COLOR_WHITE,   COLOR_WHITE,   bg);
+	init_pair(COLOR_MAGENTA, COLOR_MAGENTA, bg);
+	init_pair(COLOR_BLUE,    COLOR_BLUE,    bg);
+	init_pair(COLOR_YELLOW,  COLOR_YELLOW,  bg);
 
-	flavor[0] |= COLOR_PAIR(COLOR_GREEN) | A_BOLD;
-	flavor[1] |= COLOR_PAIR(COLOR_RED) | A_BOLD;
-	flavor[2] |= COLOR_PAIR(COLOR_CYAN) | A_BOLD;
-	flavor[3] |= COLOR_PAIR(COLOR_WHITE) | A_BOLD;
+	flavor[0] |= COLOR_PAIR(COLOR_GREEN)   | A_BOLD;
+	flavor[1] |= COLOR_PAIR(COLOR_RED)     | A_BOLD;
+	flavor[2] |= COLOR_PAIR(COLOR_CYAN)    | A_BOLD;
+	flavor[3] |= COLOR_PAIR(COLOR_WHITE)   | A_BOLD;
 	flavor[4] |= COLOR_PAIR(COLOR_MAGENTA) | A_BOLD;
-	flavor[5] |= COLOR_PAIR(COLOR_BLUE) | A_BOLD;
-	flavor[6] |= COLOR_PAIR(COLOR_YELLOW) | A_BOLD;
+	flavor[5] |= COLOR_PAIR(COLOR_BLUE)    | A_BOLD;
+	flavor[6] |= COLOR_PAIR(COLOR_YELLOW)  | A_BOLD;
     }
 #endif /* A_COLOR */
 
@@ -257,19 +267,19 @@ int last, bottom;
 		w->orientation=w->head=0;
 		if (!(ip=(short *)malloc((length+1)*sizeof (short)))) {
 		    fprintf(stderr,"%s: out of memory\n",*argv);
-		    exit(1);
+		    return EXIT_FAILURE;
 		}
 		w->xpos=ip;
 		for (x=length;--x>=0;) *ip++ = -1;
 		if (!(ip=(short *)malloc((length+1)*sizeof (short)))) {
 		    fprintf(stderr,"%s: out of memory\n",*argv);
-		    exit(1);
+		    return EXIT_FAILURE;
 		}
 		w->ypos=ip;
 		for (y=length;--y>=0;) *ip++ = -1;
     }
     if (field) {
-		register char *p;
+		register const char *p;
 		p=field;
 		for (y=bottom;--y>=0;) {
 		    for (x=COLS;--x>=0;) {
@@ -320,8 +330,9 @@ int last, bottom;
 		    switch (op->nopts) {
 		    case 0:
 				refresh();
+				curs_set(1);
 				endwin();
-				exit(0);
+				return EXIT_SUCCESS;
 		    case 1:
 				w->orientation=op->opts[0];
 				break;
@@ -338,14 +349,17 @@ int last, bottom;
     }
 }
 
-void
-onsig(int sig)
+static RETSIGTYPE
+onsig(int sig GCC_UNUSED)
 {
+	standend();
+	refresh();
+	curs_set(1);
 	endwin();
-	exit(sig);
+	exit(EXIT_FAILURE);
 }
 
-float
+static float
 ranf(void)
 {
 float rv;
